const _toString = Object.prototype.toString;
const _hasOwn = Object.prototype.hasOwnProperty;

export type Booleanish = boolean | 'true' | 'false';

export const hasOwn = (
  val: object,
  key: string | symbol
): key is keyof typeof val => _hasOwn.call(val, key);
export const toRawType = (val: unknown): string =>
  _toString.call(val).slice(8, -1);

export const isString = (val: unknown): val is string =>
  typeof val === 'string';
export const isNumber = (val: unknown): val is number =>
  typeof val === 'number';
export const isBoolean = (val: unknown): val is boolean =>
  typeof val === 'boolean';
export const isBooleanish = (val: any): val is Booleanish =>
  isBoolean(val) || val === 'true' || val === 'false';
export const isSymbol = (val: unknown): val is symbol =>
  typeof val === 'symbol';
export const isRegExp = (val: unknown) =>
  _toString.call(val) === '[object RegExp]';
export const isNil = (val: unknown): val is null | undefined => val == null;
export const isNumeric = (val: unknown) =>
  !isNaN(parseFloat(val as string)) && isFinite(val as number);
export const isDef = <T = any>(val?: T): val is T => !isNil(val);
export const isDate = (val: unknown): val is Date => val instanceof Date;
export const isFunction = (val: unknown): val is Function =>
  typeof val === 'function' || _toString.call(val) === '[object Function]';
export const isArray = Array.isArray;
export const isObject = (val: unknown): val is Record<any, any> =>
  val !== null && typeof val === 'object';
export const isPromise = <T = any>(val: unknown): val is Promise<T> =>
  isObject(val) && isFunction(val.then) && isFunction(val.catch);
// export const isPlainObject = (val: unknown): val is object => _toString.call(val) === '[object Object]'
export const isPlainObject = (val: unknown): val is object => {
  if (isObject(val) === false) return false;
  const ctor = (val as object).constructor;
  const proto = ctor.prototype;
  return (
    isFunction(ctor) &&
    isObject(proto) &&
    hasOwn(proto, 'isPrototypeOf') !== false
  );
};
export const isEmptyObject = (val: any): val is Record<any, any> => {
  if (!isPlainObject(val)) return false;
  for (const _k in val) {
    return false;
  }
  return true;
};
